
from select import select
from PyQt5.QtCore import Qt, pyqtSignal, pyqtSlot, QPoint, QModelIndex, QItemSelectionModel
from PyQt5.QtWidgets import QWidget, QAction
from PyQt5.QtWidgets import QVBoxLayout, QMenu
from PyQt5.QtWidgets import QListView


class ProcessinListView(QListView):
    selection_changed = pyqtSignal(QModelIndex)

    def __init__(self, parent=None):
        super().__init__(parent)

    def currentChanged(self, current: QModelIndex, previous: QModelIndex):
        super().currentChanged(current, previous)
        self.selection_changed.emit(current)


class ProcessinListWidget(QWidget):
    delete_item = pyqtSignal(QModelIndex, QItemSelectionModel)

    def __init__(self, parent=None):
        super().__init__(parent)
        self._current_index = None
        self.initUI()

    def initUI(self):
        self._root_layout = QVBoxLayout()

        self.processing_list = ProcessinListView(self)
        self.processing_list.setContextMenuPolicy(Qt.CustomContextMenu)
        self.processing_list.setSelectionMode(
            self.processing_list.ExtendedSelection)

        self._root_layout.addWidget(self.processing_list)

        self.contextMenu = QMenu()
        self.delete_action = QAction("Delete", self.processing_list)
        self.delete_action.triggered.connect(self.deleteClickedSlot)
        self.contextMenu.addAction(self.delete_action)
        self.processing_list.customContextMenuRequested.connect(
            self.onCustomContextMenu)

        self.setLayout(self._root_layout)

    def set_processing_list_model(self, processing_list_model):
        self.processing_list.setModel(processing_list_model)

    @pyqtSlot(QPoint)
    def onCustomContextMenu(self, point):
        self._current_index = self.processing_list.indexAt(point)
        if self._current_index.isValid():
            self.contextMenu.exec(
                self.processing_list.viewport().mapToGlobal(point))

    @pyqtSlot()
    def deleteClickedSlot(self):
        if self._current_index:
            self.delete_item.emit(self._current_index,
                                  self.processing_list.selectionModel())
            self._current_index = None



import sys

from PyQt5 import QtWidgets
from PyQt5.QtCore import Qt, pyqtSlot

from bcanalyzer.gui.controllers.basic_controller import AbstractController
from bcanalyzer.gui.views.main_window import Main_GUI

from bcanalyzer.gui.models.processing_list_model import ProcessModel
from bcanalyzer.gui.models.processing_preview_model import PreviewModel
from bcanalyzer.common.io import im_load


class MainController(AbstractController):
    def __init__(self):
        super().__init__()
        self.proj_ext = ".json"
        self._app = QtWidgets.QApplication(sys.argv)
        self._view = Main_GUI()
        self.__init_models()
        self.__init_signals()

    def run(self):
        self._view.show()
        return self._app.exec()

    def __init_models(self):
        self._processing_model = ProcessModel()
        self._view.set_processing_list_model(
            self._processing_model.process_list_model)

        self._preview_model = PreviewModel()

    def __init_signals(self):
        self._view.dragEnterEvent = self.imageLabelDragEnterEvent
        self._view.dropEvent = self.imageLabelDropEvent

        # Menu
        self._view.export_all.connect(self.export_all)
        self._view.export_renders.connect(self.export_renders)
        self._view.export_stat.connect(self.export_stat)

        # ProcessList
        self._view.processing_list_widget.processing_list.selection_changed.connect(
            self._preview_model.change_to)

        self._view.processing_list_widget.delete_item.connect(
            self._processing_model.process_list_model.removeItem)

        # processing_widget
        self._preview_model.preview.connect(
            self._view.processing_widget.updateImage)
        self._preview_model.update_values.connect(
            self._view.processing_widget.setValues)
        self._view.processing_widget.sensitivity_slider.valueChanged.connect(
            self._preview_model.set_thresh_value)
        self._view.processing_widget.window_size_slider.valueChanged.connect(
            self._preview_model.set_win_size_value)

        self._view.processing_widget.global_params_updated.connect(
            self._preview_model.upd_global_params)

        self._preview_model.save_global_params.connect(
            self._processing_model.process_list_model.updGlobalData)

        self._view.processing_widget.save_local_params.connect(
            self._preview_model.upd_local_param)
        self._preview_model.save_local_params.connect(
            self._processing_model.process_list_model.updLocalData)
        self._view.processing_widget.remove_local_params.connect(
            self._preview_model.remove_local_param)
        self._preview_model.remove_local_params.connect(
            self._processing_model.process_list_model.removeLocalData)

    def __load_screen(self, next_screen: int):
        raise Exception(f"Can't find screen with num={next_screen}")

    def imageLabelDragEnterEvent(self, e):
        if e.mimeData().hasUrls():
            e.accept()
        else:
            e.ignore()

    def imageLabelDropEvent(self, event):
        files = [u.toLocalFile() for u in event.mimeData().urls()]
        corrupted_files = []
        for f in files:
            if im_load(f) is not None:
                self._processing_model.process_list_model.add_item(f)
            else:
                corrupted_files.append(f)
        if len(corrupted_files) > 0:

            message_list_text = ""
            for f in corrupted_files:
                message_list_text += f
                message_list_text += "\n"
            QtWidgets.QMessageBox.warning(
                self._view, "Warning", f"The next images is corrupted: \n {message_list_text}")

    @pyqtSlot(str, str)
    def export_all(self, target_folder: str, target_file: str):
        row_count, process_sig, terminator_slot = self._processing_model.process_data(
            target_folder, target_file)
        if row_count == 0:
            return
        pd = self._view.show_progress_dialog(
            "Exporting all data", 0, row_count)

        process_sig.connect(pd.setValue)
        pd.canceled.connect(terminator_slot, Qt.DirectConnection)
        pd.show()

    @pyqtSlot(str)
    def export_renders(self, target_folder: str):
        row_count, process_sig, terminator_slot = self._processing_model.process_data(
            target_folder, None)
        if row_count == 0:
            return
        pd = self._view.show_progress_dialog(
            "Rendering", 0, row_count)

        process_sig.connect(pd.setValue)
        pd.canceled.connect(terminator_slot, Qt.DirectConnection)
        pd.show()

    @pyqtSlot(str)
    def export_stat(self, target_file: str):
        row_count, process_sig, terminator_slot = self._processing_model.process_data(
            None, target_file)
        if row_count == 0:
            return
        pd = self._view.show_progress_dialog(
            "Preparing statistics", 0, row_count)

        process_sig.connect(pd.setValue)
        pd.canceled.connect(terminator_slot, Qt.DirectConnection)
        pd.show()



from PyQt5.QtCore import Qt, pyqtSlot, pyqtSignal
from PyQt5.QtWidgets import QPushButton, QWidget, QSizePolicy
from PyQt5.QtWidgets import QVBoxLayout, QHBoxLayout, QGroupBox
from PyQt5.QtWidgets import QLabel, QCheckBox, QComboBox
from PyQt5.QtGui import QPixmap, QImage, QResizeEvent

from bcanalyzer.gui.widgets.extended_slider_widget import ExtendedSliderWidget
import logging

logger = logging.Logger(__file__)


class ProcessingPreviewWidget(QWidget):
    global_params_updated = pyqtSignal(dict)
    save_local_params = pyqtSignal(dict)
    remove_local_params = pyqtSignal()

    def __init__(self, parent=None):
        super().__init__(parent)
        # self._processing_model = processing_model
        self._curent_image = None
        self.initUI()

    def initUI(self):
        self._root_layout = QVBoxLayout()
        # 1. Common settings
        self.common_settings_group = QGroupBox(
            "Common settings")
        self.common_settings_group_layout = QVBoxLayout()
        self.common_settings_group.setLayout(self.common_settings_group_layout)
        self._root_layout.addWidget(self.common_settings_group)

        # 1.1 Channel Selection
        self.channel_group = QGroupBox("Select color channels for processing")
        self.channel_group_layout = QHBoxLayout()

        self.red_channel_check_box = QCheckBox()
        self.red_channel_check_box.setText("RED")
        self.red_channel_check_box.setChecked(True)
        self.channel_group_layout.addWidget(self.red_channel_check_box)
        self.red_channel_check_box.released.connect(
            self.global_param_wigdet_released)

        self.green_channel_check_box = QCheckBox()
        self.green_channel_check_box.setText("GREEN")
        self.green_channel_check_box.setChecked(True)
        self.channel_group_layout.addWidget(self.green_channel_check_box)
        self.green_channel_check_box.released.connect(
            self.global_param_wigdet_released)

        self.blue_channel_check_box = QCheckBox()
        self.blue_channel_check_box.setText("BLUE")
        self.blue_channel_check_box.setChecked(True)
        self.channel_group_layout.addWidget(self.blue_channel_check_box)
        self.blue_channel_check_box.released.connect(
            self.global_param_wigdet_released)

        self.channel_group.setLayout(self.channel_group_layout)

        self.common_settings_group_layout.addWidget(self.channel_group)

        # 1.2 Global processing settings
        self._top_gl_settings_layout = QHBoxLayout()
        self.common_settings_group_layout.addLayout(
            self._top_gl_settings_layout)

        # 1.2.1 Use BG removing
        self.do_bg_removing = QCheckBox(self)
        self.do_bg_removing.setObjectName("do_bg_removing")
        self.do_bg_removing.setText("Apply BG removal")
        self._top_gl_settings_layout.addWidget(self.do_bg_removing)
        self.do_bg_removing.released.connect(
            self.global_param_wigdet_released)

        # 1.2.2 Use abs threshold

        self.use_abs_threshold = QCheckBox(self)
        self.use_abs_threshold.setObjectName("use_abs_threshhold")
        self.use_abs_threshold.setText("Freeze threshold")
        self._top_gl_settings_layout.addWidget(self.use_abs_threshold)
        self.use_abs_threshold.stateChanged.connect(
            self.use_abs_threshold_state_changed)
        self.use_abs_threshold.released.connect(
            self.global_param_wigdet_released)

        # 1.2.3 Single object

        self.one_contour_only = QCheckBox(self)
        self.one_contour_only.setObjectName("one_contour_only")
        self.one_contour_only.setText("Single oblect segmentation")
        self._top_gl_settings_layout.addWidget(self.one_contour_only)
        self.one_contour_only.released.connect(
            self.global_param_wigdet_released)

        # 1.2.4 Automatic thresholding
        self.do_otsu_thresholding = QComboBox(self)
        self.do_otsu_thresholding.addItems(["Disable",
                                            "Otsu",
                                            "SDD lower",
                                            "SDD median",
                                            "SDD upper"])
        self.do_otsu_thresholding.setObjectName("do_otsu_thresholding")
        #self.do_otsu_thresholding.setText("Automatic threshold")
        self._top_gl_settings_layout.addWidget(QLabel("Auto sensitivity:"))
        self._top_gl_settings_layout.addWidget(self.do_otsu_thresholding)
        self.do_otsu_thresholding.currentIndexChanged.connect(
            self.do_otsu_thresholding_state_changed)
        self.do_otsu_thresholding.currentIndexChanged.connect(
            self.global_param_wigdet_released)

        # 2. Preview

        self._image_label = QLabel(self)
        self._image_label.setText("")
        self._image_label.setAlignment(Qt.AlignHCenter)
        self._image_label.setObjectName("image_label")
        self._image_label.setSizePolicy(
            QSizePolicy.Expanding, QSizePolicy.Expanding)
        self._root_layout.addWidget(self._image_label)

        # 3. Local settings

        self.local_settings_group_layout = QVBoxLayout()
        self.local_settings_group = QGroupBox()
        self.local_settings_group.setTitle("Algorithm parameters")
        self.local_settings_group.setLayout(self.local_settings_group_layout)
        self._root_layout.addWidget(self.local_settings_group)

        self.sensitivity_slider = ExtendedSliderWidget("Sensitivity")
        self.sensitivity_slider.set_range(0, 100)
        self.window_size_slider = ExtendedSliderWidget("Resolution")
        self.window_size_slider.set_range(10, 100)
        self.window_size_slider.setSingleStep(1)
        self.window_size_slider.set_integer(True)

        self.local_settings_group_layout.addWidget(self.sensitivity_slider)
        self.local_settings_group_layout.addWidget(self.window_size_slider)

        self.button_layout = QHBoxLayout()
        self.local_settings_group_layout.addLayout(self.button_layout)
        self.accept4all_button = QPushButton("Apply to all")
        self.accept4all_button.clicked.connect(self.accept4all_button_clicked)
        self.accept4current_button = QPushButton("Apply to current")
        self.accept4current_button.clicked.connect(
            self.accept4current_button_clicked)
        self.cancel_individual_button = QPushButton("Remove individual")
        self.cancel_individual_button.clicked.connect(
            self.cancel_individual_button_clicked)
        self.button_layout.addWidget(self.accept4all_button)
        self.button_layout.addWidget(self.accept4current_button)
        self.button_layout.addWidget(self.cancel_individual_button)

        # self._root_layout.addStretch()
        self.setLayout(self._root_layout)

    def updateImage(self, image: QImage):
        self._curent_image = image.scaled(
            1920, 1080, Qt.KeepAspectRatio)
        self.__update_image()

    def __update_image(self):
        if self._curent_image is None:
            return
        try:
            image = self._curent_image.scaled(
                int(self._image_label.width()*0.95),
                int(self._image_label.height()*0.95),
                Qt.KeepAspectRatio
            )
            self._image_label.setPixmap(QPixmap(image))
        except Exception as e:
            print(str(e))

    def resizeEvent(self,  e: QResizeEvent):
        super().resizeEvent(e)
        self.__update_image()

    @pyqtSlot(bool)
    def accept4all_button_clicked(self, checked: bool):
        glob_param_values = {}
        glob_param_values['thre'] = self.sensitivity_slider.value()
        glob_param_values['win_size'] = self.window_size_slider.value()
        self.global_params_updated.emit(glob_param_values)

    @pyqtSlot(bool)
    def accept4current_button_clicked(self, checked: bool):
        param_values = {}
        param_values['thre'] = self.sensitivity_slider.value()
        param_values['win_size'] = self.window_size_slider.value()
        self.save_local_params.emit(param_values)

    @pyqtSlot(bool)
    def cancel_individual_button_clicked(self, checked: bool):
        self.remove_local_params.emit()

    @pyqtSlot(int)
    def use_abs_threshold_state_changed(self, state: int):
        if state == Qt.CheckState.Checked:
            self.do_otsu_thresholding.setCurrentText('Disable')

    @pyqtSlot(int)
    def do_otsu_thresholding_state_changed(self, index: int):
        if index != 0:
            self.use_abs_threshold.setChecked(False)

    @pyqtSlot()
    def global_param_wigdet_released(self, v=None):
        glob_param_values = {}
        glob_param_values['channel_r'] = self.red_channel_check_box.isChecked()
        glob_param_values['channel_g'] = self.green_channel_check_box.isChecked()
        glob_param_values['channel_b'] = self.blue_channel_check_box.isChecked()
        glob_param_values['is_single_object'] = self.one_contour_only.isChecked()
        glob_param_values['do_bg_removing'] = self.do_bg_removing.isChecked()

        glob_param_values['use_abs_threshhold'] = self.use_abs_threshold.isChecked()
        glob_param_values['do_otsu_thresholding'] = (
            self.do_otsu_thresholding.currentText())

        self.global_params_updated.emit(glob_param_values)

    @ pyqtSlot(dict)
    def setValues(self, values):

        sensitivity = values.get('thre', 50)
        self.sensitivity_slider.set_value(sensitivity)

        window_size = values.get('win_size', 20)
        self.window_size_slider.set_value(window_size)

        channel_r = values.get('channel_r', True)
        self.red_channel_check_box.setChecked(channel_r)

        channel_g = values.get('channel_g', True)
        self.green_channel_check_box.setChecked(channel_g)

        channel_b = values.get('channel_b', True)
        self.blue_channel_check_box.setChecked(channel_b)

        is_single_object = values.get('is_single_object', False)
        self.one_contour_only.setChecked(is_single_object)

        thre_abs = values.get("thre_abs", '')
        self.use_abs_threshold.setText(
            f"Freeze threshold ({thre_abs/(255):.4f})")

        do_bg_removing = values.get('do_bg_removing', False)
        self.do_bg_removing.setChecked(do_bg_removing)

        do_otsu_thresholding = values.get('do_otsu_thresholding', 'Disable')
        self.do_otsu_thresholding.setCurrentText(do_otsu_thresholding)



import numpy as np
import logging
from PyQt5.QtCore import Qt, pyqtSlot, pyqtSignal, QModelIndex, QObject
from PyQt5.QtGui import QImage

from bcanalyzer.image_processing.image_processor import process_image


class PreviewModel(QObject):
    preview = pyqtSignal(QImage)
    update_values = pyqtSignal(dict)

    save_local_params = pyqtSignal(QModelIndex, dict)
    save_global_params = pyqtSignal(dict)
    remove_local_params = pyqtSignal(QModelIndex)

    def __init__(self):
        super().__init__()
        self._im_url = None
        self._params = None
        self._current_model_index = None

    def update_processing(self, is_user=False):
        if self._current_model_index is None:
            return
        global_params, info = self._current_model_index.data(Qt.UserRole)
        im_url = info.url

        # self.update_values.emit(self._params)
        try:
            img_np, res, res_meta = process_image(
                im_url, self._params, force_q_th=is_user)
        except:
            logging.exception(f"Cannot process {im_url}", exc_info=False)
            return
        self._params['thre_abs'] = res_meta["thre_abs"]
        self._params['thre'] = res_meta["thre"]
        self.update_values.emit(self._params)

        img_np = np.require(img_np, np.uint8, 'C')
        height, width, channel = img_np.shape
        bytesPerLine = 3 * width
        convertToQtFormat = QImage(
            img_np.data, width, height, bytesPerLine, QImage.Format_BGR888)
        convertToQtFormat.ndarray = img_np
        self.preview.emit(convertToQtFormat)

    def _update_params(self, item: QModelIndex):
        global_params, info = item.data(Qt.UserRole)
        params = {}
        params.update(global_params)
        if info.alg_params is not None:
            params['thre'] = info.alg_params['thre']
            params['win_size'] = info.alg_params['win_size']
        self._params = params

    @ pyqtSlot(QModelIndex)
    def change_to(self, item: QModelIndex):
        self._current_model_index = item
        self._update_params(self._current_model_index)
        self.update_processing()

    @ pyqtSlot(dict)
    def upd_global_params(self, value: dict) -> bool:
        if self._params is None:
            return False
        self._params.update(value)
        self.update_processing(is_user=True)
        self.save_global_params.emit(self._params)

    @ pyqtSlot(float)
    def set_thresh_value(self, value: float):
        if self._params is None:
            return False
        self._params['thre'] = value
        self.update_processing(is_user=True)

    @ pyqtSlot(float)
    def set_win_size_value(self, value: int):
        if self._params is None:
            return False
        self._params['win_size'] = value
        self.update_processing(is_user=True)

    def upd_local_param(self, params):
        if self._current_model_index is None:
            return False
        self.save_local_params.emit(self._current_model_index, params)

    @ pyqtSlot()
    def remove_local_param(self):
        if self._current_model_index is None:
            return False
        self.remove_local_params.emit(self._current_model_index)
        self._update_params(self._current_model_index)
        self.update_processing()



import os
import sys
import appdirs

from bcanalyzer.common.const import *


def config_file_path():
    return os.path.join(appdirs.user_config_dir(APP_NAME, False), "settings.json")


if __name__ == "__main__":
    sys.path.append(os.path.abspath(os.path.join(
        os.path.dirname(__file__), '..', '..')))

    print(config_file_path())



import os
import cv2
import numpy as np


def im_load(name):
    f = open(name, "rb")
    chunk = bytearray(f.read())
    chunk_arr = np.frombuffer(chunk, dtype=np.uint8)  #
    img = cv2.imdecode(chunk_arr, cv2.IMREAD_UNCHANGED)
    if len(img.shape) == 3 and img.shape[2] == 4:
        img = img[:, :, :3]
    f.close()

    return img


def im_save(target_path: str, image_np: np.ndarray):
    filename, file_extension = os.path.splitext(target_path)
    is_success, im_buf_arr = cv2.imencode(file_extension, image_np)
    return im_buf_arr.tofile(target_path)




import os
from shutil import ExecError
import filetype
import pandas as pd
import numpy as np
import tempfile
import queue
import logging
import typing
import copy
import logging
from PyQt5 import QtCore
from PyQt5.QtCore import QObject, QModelIndex, QThread, QObject, pyqtSlot, QItemSelectionModel
from PyQt5.QtGui import QPixmap
from bcanalyzer.image_processing.image_processor import process_image
from bcanalyzer.common.io import im_save


def build_target_path(origin_path, target_dir, suffix):
    basename = os.path.basename(origin_path)
    name, ext = os.path.splitext(basename)
    target_path = f"{name}_{suffix}{ext}"
    target_path = os.path.join(target_dir, target_path)
    return target_path


class Worker(QObject):
    progress = QtCore.pyqtSignal(int)
    finished = QtCore.pyqtSignal()

    def __init__(self, target_path: str, target_file: str, global_params: dict, data: pd.DataFrame):
        super().__init__()
        self.threads_num = 4
        self.base_path = target_path
        self.report_path = target_file
        self.data = data
        self._is_not_canceled = True
        self._global_params = global_params

    def run(self):
        if self.base_path is not None:
            os.makedirs(self.base_path, exist_ok=True)

        column_names = ['file_name',
                        'abs_threshold',
                        'Non-selected area, px',
                        'Selected area, px',
                        'Non-selected area, %',
                        'Selected area, %',
                        ]
        self._report = pd.DataFrame(columns=column_names)

        for i, row in self.data.iterrows():
            params = {}
            params = copy.deepcopy(self._global_params)
            # params['thre'] = self._global_params['thre']
            # params['win_size'] = self._global_params['win_size']
            # params['channel_r'] = self._global_params['channel_r']
            # params['channel_g'] = self._global_params['channel_g']
            # params['channel_b'] = self._global_params['channel_b']
            # params['is_single_object'] = self._global_params['is_single_object']
            # params['thre_abs'] = self._global_params['thre_abs']
            # params['use_abs_threshhold'] = self._global_params['use_abs_threshhold']
            # params['do_bg_removing'] = self._global_params['do_bg_removing']
            # params['do_otsu_thresholding'] = self._global_params['do_otsu_thresholding']

            if row.alg_params is not None:
                params['thre'] = row.alg_params['thre']
                params['win_size'] = row.alg_params['win_size']

            try:
                img_np, mask, res_meta = process_image(row.url, params)
                basename = os.path.basename(row.url)
                if self.base_path is not None:
                    img_path = build_target_path(
                        row.url, self.base_path, "RENDER")
                    im_save(img_path, img_np)
                    img_path = build_target_path(
                        row.url, self.base_path, "MASK")
                    mask4save = mask.copy()
                    if mask4save.max() == 1:
                        mask4save *= 255
                    im_save(img_path, mask4save)

                above_threshold_px = np.count_nonzero(mask)
                below_threshold_px = mask.size - above_threshold_px
                data_row = {
                    'file_name': basename,
                    'abs_threshold': res_meta['thre_abs'] / 255.0,
                    'Non-selected area, px': below_threshold_px,
                    'Selected area, px': above_threshold_px,
                    'Non-selected area, %': below_threshold_px / mask.size,
                    'Selected area, %': above_threshold_px / mask.size,
                }
                self._report = self._report.append(data_row, ignore_index=True)
            except:
                logging.exception(f"Cannot process {row.url}", exc_info=False)

            if self._is_not_canceled:
                self.progress.emit(i+1)
            else:
                break
        if self.report_path is not None:
            self._report.to_csv(self.report_path)
        self.finished.emit()

    def __process(self, info):
        index, row = info

        img_np, result = process_image(row.url)

        #cv2.imwrite(os.path.join(self.base_path, row.label), img_np)
        logging.debug(f"Exp was completed {row.label}")
        self.queue.task_done()
        self.result.emit(index, result)
        self.progress.emit(index)
        self.sem.release()

    @pyqtSlot()
    def cancel_processing(self):
        logging.debug("Cancel processing by User")
        self._is_not_canceled = False


class ProcessListModel(QtCore.QAbstractListModel):
    def __init__(self,
                 parent: QObject = None) -> None:
        super().__init__(parent)
        self._cache_dir = tempfile.TemporaryDirectory()
        self._global_params = {}
        self._global_params['thre'] = 50
        self._global_params['thre_abs'] = None
        self._global_params['win_size'] = 32
        self._global_params['channel_r'] = True
        self._global_params['channel_g'] = True
        self._global_params['channel_b'] = True
        self._global_params['is_single_object'] = False
        self._global_params['use_abs_threshhold'] = False
        self._global_params['do_bg_removing'] = False
        self._global_params['do_otsu_thresholding'] = "Disable"

        column_names = ['url',
                        'label',
                        'is_processed',
                        'alg_params',
                        'result']
        self._data = pd.DataFrame(columns=column_names)

    def add_item(self, url):
        self.beginInsertRows(QModelIndex(), self.rowCount(), self.rowCount())
        if url in self._data['url'].values:
            return
        if not filetype.is_image(url):
            return
        data_row = {
            'url': url,
            'label': os.path.basename(url),
            'is_processed': False,
            'alg_params': None,
            'result': None
        }
        self._data = self._data.append(data_row, ignore_index=True)
        self.endInsertRows()

    def data(self, index: QModelIndex, role: int = ...):
        row = index.row()
        value = self._data.iloc[row]
        if role == QtCore.Qt.DisplayRole:
            is_unique_params = "*" if value.alg_params is not None else ""
            return is_unique_params + value.label
        # elif role == QtCore.Qt.DecorationRole:
        #    pixmap = QtGui.QPixmap(26, 26)
        #    if not value.is_processed:
        #        pixmap.fill(QtGui.QColor('red'))
        #    else:
        #        pixmap.fill(QtGui.QColor('green'))
        #    return QtGui.QIcon(pixmap)
        elif role == QtCore.Qt.UserRole:
            row = index.row()
            return self._global_params, value

        return None

    def reverse_processed(self, index: int):
        self._data.iloc[index].is_processed = not self._data.iloc[index].is_processed

    def is_processed(self, index: int):
        return self._data.iloc[index].is_processed

    def rowCount(self, parent: QModelIndex = ...) -> int:
        return len(self._data.index)

    def get_raw_data(self):
        return self._data

    def get_image_path(self, index: int) -> typing.Union[str, None]:
        return self._data.iloc[index].url

    def on_result_listener(self, index: int, result):
        self._data.iloc[index].url = result

    @pyqtSlot(QModelIndex, dict)
    def updLocalData(self, index: QModelIndex, value: dict):
        row = index.row()
        # = {'thre': 0.7}  #
        self._data.at[row, "alg_params"] = value
        self.dataChanged.emit(index, index)

    @pyqtSlot(QModelIndex)
    def removeLocalData(self, index: QModelIndex):
        row = index.row()
        self._data.at[row, "alg_params"] = None
        self.dataChanged.emit(index, index)

    @pyqtSlot(dict)
    def updGlobalData(self, value: dict):
        self._global_params.update(value)

    @pyqtSlot(QModelIndex, QItemSelectionModel)
    def removeItem(self, index: QModelIndex, selection_model: QItemSelectionModel):
        if selection_model.hasSelection():
            self.beginRemoveRows(index,
                                 selection_model.selectedRows()[0].row(),
                                 selection_model.selectedRows()[-1].row())
            list_to_delete = []
            for row_ind in selection_model.selectedRows():
                list_to_delete.append(self._data.index[row_ind.row()])
            self._data.drop(list_to_delete, inplace=True)
        else:
            self.beginRemoveRows(index, index.row(), index.row())
            self._data.drop(self._data.index[index.row()], inplace=True)
            self.endRemoveRows()


class ProcessModel(QObject):
    def __init__(self):
        super().__init__()
        self.process_list_model: ProcessListModel = ProcessListModel()

    def get_processed_image(self, index: int) -> typing.Union[QPixmap, None]:
        """
        :param index:
        :type index:
        :param tab_index: Result of QTabWidget.currentIndex() . tab_index == 0 is first tab
        :type tab_index:
        :return:
        :rtype:
        """
        processed_path = self.process_list_model.get_image_path(
            self, index)
        if processed_path:
            qpm = QPixmap(processed_path)
            return qpm
        return None

    def process_data(self, target_folder=None, target_file=None):
        self._q = queue.Queue()
        self._thread = QThread()
        self._worker = Worker(
            target_folder,
            target_file,
            self.process_list_model._global_params,
            self.process_list_model.get_raw_data())
        self._worker.moveToThread(self._thread)

        # connect_signals_and_slots
        self._thread.started.connect(self._worker.run)
        self._worker.finished.connect(self._thread.quit)
        self._thread.finished
        self._thread.start()

        return self.process_list_model.rowCount(), self._worker.progress, self._worker.cancel_processing
